﻿using Hotline.Caching.Interfaces;
using Hotline.Caching.Services;
using Hotline.CallCenter.Calls;
using Hotline.Realtimes;
using Hotline.Share.Dtos.Realtime;
using Hotline.Share.Enums.CallCenter;
using Hotline.Share.Notifications;
using MediatR;

namespace Hotline.Application.Handlers.CallCenter.FlowControl
{
    public class AnswerViisitorToExtNotificationHandler:INotificationHandler<AnswerViisitorToExtNotification>
    {
        private readonly ICallRepository _callRepository;
        private readonly ICallDetailRepository _callDetailRepository;
        private readonly ICallCacheManager _callCacheManager;
        private readonly IRealtimeService _realtimeService;


        public AnswerViisitorToExtNotificationHandler(ICallRepository callRepository, ICallDetailRepository callDetailRepository,ICallCacheManager callCacheManager,IRealtimeService realtimeService)
        {
            _callRepository = callRepository;
            _callDetailRepository = callDetailRepository;
            _callCacheManager = callCacheManager;
            _realtimeService = realtimeService;
        }

        public async Task Handle(AnswerViisitorToExtNotification notification, CancellationToken cancellationToken)
        {
            var model = await _callRepository.GetAsync(
                x => x.ConversationId == notification.Visitor.Id &&
                     x.FromNo == notification.Visitor.From && x.CreationTime >= DateTime.Now.AddHours(-2),true,x=>x.CreationTime, cancellationToken);
            if (model != null)
            {
                model.CallStatus = ECallStatus.Answer;
                await _callRepository.UpdateAsync(model, cancellationToken);
                var detail = new CallDetail()
                {
                    CallId = model.Id,
                    CallStatus = ECallStatus.Answer,
                    OMCallId = notification.Visitor.CallId,
                    ConversationId = notification.Visitor.Id,
                    EventName = notification.Attribute,
                    AnswerNo = notification.TelNo,
                    FromNo = notification.Visitor.From,
                    ToNo = notification.Visitor.To,
                };
                await _callDetailRepository.AddAsync(detail, cancellationToken);


                //处理排队缓存数据
                _callCacheManager.RemoveCallCache(model.Id);
                await _realtimeService.CallQueueAsync(_callCacheManager.GetCallQueueList(), cancellationToken);
            }
        }
    }
}
